﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using Outlook = Microsoft.Office.Interop.Outlook;
using Office = Microsoft.Office.Core;
using System.Windows.Forms;
using Word = Microsoft.Office.Interop.Word;
using System.IO;

namespace OutlookToWord
{
    public partial class ThisAddIn
    {
        private Office.CommandBar newToolBar;
        private Office.CommandBarButton exportButton;
        private Outlook.Explorers selectExplorers;
        private FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog(); 

        private void ThisAddIn_Startup(object sender, System.EventArgs e)
        {
            selectExplorers = this.Application.Explorers;
            selectExplorers.NewExplorer += new Outlook.ExplorersEvents_NewExplorerEventHandler(newExplorer_Event);
            AddToolbar();
        }

        private void newExplorer_Event(Outlook.Explorer new_Explorer)
        {
            ((Outlook._Explorer)new_Explorer).Activate();
            newToolBar = null;
            AddToolbar();
        }

        private void AddToolbar()
        {
            if (newToolBar == null)
            {
                Office.CommandBars cmdBars = this.Application.ActiveExplorer().CommandBars;
                newToolBar = cmdBars.Add("NewToolBar", Office.MsoBarPosition.msoBarTop, false, true);
            }
            try
            {
                Office.CommandBarButton btExportToWord = (Office.CommandBarButton)newToolBar.Controls.Add(1, missing, missing, missing, missing);
                btExportToWord.Style = Office.MsoButtonStyle.msoButtonCaption;
                btExportToWord.Caption = "Export to Word";
                btExportToWord.Tag = "Export current mail to word";
                if (this.exportButton == null)
                {
                    this.exportButton = btExportToWord;
                    exportButton.Click += new Office._CommandBarButtonEvents_ClickEventHandler(exportButton_Click);
                }
                newToolBar.Visible = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        /// <summary>
        /// Create export file name from a string.
        /// </summary>
        /// <param name="sPath"></param>
        /// <param name="sFileName"></param>
        /// <returns></returns>
        private string CreateFileName(string sPath, string sFileName)
        {
            // Remove unsupport charts for file name.
            string sRst = sFileName.Replace("\\", "");
            sRst = sRst.Replace("/", "");
            sRst = sRst.Replace(":", "");
            sRst = sRst.Replace("*", "");
            sRst = sRst.Replace("?", "");
            sRst = sRst.Replace("\"\"", "");
            sRst = sRst.Replace("<", "");
            sRst = sRst.Replace(">", "");
            sRst = sRst.Replace("|", "");

            if (sRst.Length > 100)
            {
                sRst = sRst.Substring(0, 100);
            }

            // Make sure the file name is unique.
            int i = 1;
            if (File.Exists(string.Concat(sPath, sRst, ".docx")))
            {
                while (true)
                {
                    if (File.Exists(string.Concat(sPath, sRst, i.ToString(), ".docx")))
                    {
                        i++;
                    }
                    else
                    {
                        sRst += i.ToString();
                        break;
                    }
                }
            }

            // Return *.docx file name.
            return string.Concat(sPath, sRst, ".docx");
        }

        void exportButton_Click(Office.CommandBarButton ctrl, ref bool cancel)
        {
            object sFileName;
            string sPath = string.Empty;

            // Get mail export path.
            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
            {
                sPath = folderBrowserDialog.SelectedPath;

                // If the selected path is a folder, add '\' at the end of the path string.
                if (sPath != Path.GetPathRoot(sPath))
                {
                    sPath += "\\";
                }
            }
            else
            {
                return;
            }

            Word.Application app = new Word.Application();
            Word.Document doc = null;
            object unknow = Type.Missing;
            object format = Word.WdSaveFormat.wdFormatDocumentDefault;
            Outlook.Explorer activeExplorer = this.Application.Explorers.Application.ActiveExplorer();

            try
            {
                // Export all selected mail items to word.
                foreach (object selectedItem in activeExplorer.Selection)
                {
                    Outlook.MailItem mailItem = selectedItem as Outlook.MailItem;
                    if (mailItem != null)
                    {
                        sFileName = CreateFileName(sPath, mailItem.Subject);
                        //doc = app.Documents.Add(ref unknow, ref unknow, ref unknow, ref unknow);
                        //doc.Content.Text = mailItem.Body;
                        //doc.Paragraphs.Last.Range.Text = mailItem.Body;

                        Outlook.Inspector inspector = mailItem.GetInspector;
                        doc = (Word.Document)inspector.WordEditor;
                        //mailItem.SaveAs(sFileName.ToString(), Outlook.OlSaveAsType.olDoc);

                        doc.SaveAs(ref sFileName, ref format, ref unknow, ref unknow, ref unknow,
                            ref unknow, ref unknow, ref unknow, ref unknow, ref unknow, ref unknow,
                            ref unknow, ref unknow, ref unknow, ref unknow, ref unknow);
                        //doc.Close(ref unknow, ref unknow, ref unknow);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                if (app != null)
                {
                    app.Quit(ref unknow, ref unknow, ref unknow);
                }
            }
        }

        private void ThisAddIn_Shutdown(object sender, System.EventArgs e)
        {
        }

        #region VSTO generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(ThisAddIn_Startup);
            this.Shutdown += new System.EventHandler(ThisAddIn_Shutdown);
        }

        #endregion
    }
}